module PowerTrackV2
  module Errors
    class BadRequestError           < StandardError; end
    class UnauthorizedError         < StandardError; end
    class NoSuchResourceError       < StandardError; end
    class UnprocessableEntityError  < StandardError; end
    class RateLimitedError          < StandardError; end
    class ServiceUnavailableError   < StandardError; end
    class UnexpectedStatusError     < StandardError; end

    class InvalidRuleError          < StandardError; end
    class TooManyRulesError         < StandardError; end

    ERROR_MAP = {
      400 => BadRequestError,
      401 => UnauthorizedError,
      404 => NoSuchResourceError,
      422 => UnprocessableEntityError,
      429 => RateLimitedError,
      503 => ServiceUnavailableError
    }

    def self.raise_error_for_code(code, message)
      raise ERROR_MAP[code], message if ERROR_MAP[code]
      raise UnexpectedStatusError, message
    end
  end
end
