require 'base64'
require 'httpclient'
require 'exponential_backoff'

module PowerTrackV2
  module HTTP
    def self.auth_digest(username, password)
      ::Base64.strict_encode64("#{username}:#{password}")
    end

    def self.auth_header(auth_digest)
      {'Authorization' => "Basic #{auth_digest}"}
    end

    def self.headers(headers = {})
      username = ENV['POWERTRACK_USERNAME']
      password = ENV['POWERTRACK_PASSWORD']

      digest = auth_digest(username, password)
      headers.merge auth_header(digest)
    end

    def self.get(url, headers={})
      ::HTTPClient.get(url, nil, headers)
    end

    def self.post(url, body, headers={})
      ::HTTPClient.post(url, body, headers)
    end

    def self.stream(url, headers = {}, opts = { ignore_heartbeat: true }, &block)
      http = ::HTTPClient.new
      backoff = ::ExponentialBackoff.new(1, 120)

      while true
        begin
          connection = connect(url, headers)
          stream_from(connection, opts, backoff, &block)
        rescue StandardError => e
          sleep(backoff.next_interval)
        end
      end
    end

    private

    def self.connect(url, headers)
      http = ::HTTPClient.new
      http.get_async(url, nil, headers)
    end

    def self.stream_from(connection, opts, backoff, &block)
      ignore_heartbeat = opts[:ignore_heartbeat]
      content = connection.pop.content
      backoff.clear
      until connection.finished?
        next_response = content.gets
        block.call(next_response) unless ignore_heartbeat && next_response == "\r\n"
      end
    end
  end
end
